# -*- coding: UTF-8 -*-

from random import randint, random

import arrow


class Clock:
    """
    A clock to be used by a ``Bot``.
    """

    def __init__(self, cfg):
        self.tz = cfg.get("bot", "timezone")

    def time_to_sleep(self):
        """
        Return True if it's time to sleep.
        """
        h, m = self.clock()
        day_ts = h * 60 + m

        # Sleep at some point between 22:00 and 23:59
        sleeping_time = randint(22 * 60, 24 * 60 - 1)
        # Wake up at some point between 9:00 and 11:00
        wakeup_time = randint(9 * 60, 11 * 60)

        return day_ts > sleeping_time or day_ts < wakeup_time

    def weekend(self):
        """
        Return ``True`` if it's the weekend.
        """
        # Monday = 0
        # ...
        # Saturday = 5
        # Sunday = 6
        return self.now().weekday() in (5, 6)

    def time_to_chill(self):
        """
        Return ``True`` if the bot currently have some time to chill; i.e. it's
        free to tweet.
        """
        h, _ = self.clock()

        # Our bot has less time to chill during work hours
        if not self.weekend() and (10 <= h <= 11 or 14 <= h <= 17):
            return random() > 0.7

        return random() > 0.5

    def now(self):
        """
        Return the current time.
        """
        return arrow.utcnow().to(self.tz)

    def clock(self):
        """
        Return a tuple of ``(hours, minutes)`` that represent the time in the
        bot's timezone.
        """
        tm = self.now().time()
        return tm.hour, tm.minute



# -*- coding: UTF-8 -*-

from random import random

from blabbr import text as tx


class Generator:
    """
    A random tweets generator.
    """

    def __init__(self, model):
        self.model = model

    def tweets(self, count=20, min_length=50):
        n = 0
        while n < count:
            t = self.model.make_tweet()
            if not t:
                continue

            if not self._tweet_ok(t, min_length):
                continue

            yield self.decorate_tweet(t)
            n += 1

    def decorate_tweet(self, text):
        # Remove spaces around punctuation. This may vary per language
        text = tx.fix_punctuation(text)

        if len(text) < 138 and text[-1] not in "?!.":
            if random() > 0.9:
                bang = "!" if random() > 0.1 else "!!"
                text = "%s %s" % (text, bang)
        return text

    def tweet(self, min_length=50):
        for t in self.tweets(1, min_length=min_length):
            return t

    def _tweet_ok(self, t, min_length=50):
        _forbidden_terms = (
            # FR
            "tuer", "mort", "suicid", "bombe", "encul",
        )

        if len(t) < min_length:
            return False

        lt = t.lower()

        for term in _forbidden_terms:
            if term in lt:
                return False

        return True



# -*- coding: UTF-8 -*-

import re
import json
import os.path
from collections import deque

import nltk
from nltk.tokenize import TweetTokenizer
import markovify
import markovify.text
from markovify.chain import Chain

from blabbr.twitter import TwitterClient
from blabbr.logging import getLogger
from blabbr import text as tx

# Simple format versionning
DUMP_FMT_VERSION = 1


# from https://github.com/jsvine/markovify#extending-markovifytext
class POSifiedText(markovify.Text):
    def __init__(self, input_text, state_size=2, chain=None):
        self.tokenizer = TweetTokenizer(reduce_len=True)

        self.tag_sep = "@::@"

        # Circumvent some limitations of markovify by allowing one to create a
        # POSifiedText from a markovify.Text instance
        if isinstance(input_text, markovify.Text):
            m = input_text
            self.input_text = m.input_text
            self.rejoined_text = m.rejoined_text
            self.chain = m.chain
        else:
            super().__init__(input_text, state_size, chain)

    def word_split(self, sentence):
        words = self.tokenizer.tokenize(sentence)
        words = [self.tag_sep.join((tag, word))
                 for word, tag in nltk.pos_tag(words) if word]
        return words

    def word_join(self, words):
        return " ".join(word.split(self.tag_sep, 1)[1] for word in words)


class NewlinePOSifiedText(POSifiedText):
    def sentence_split(self, text):
        return text.split("\n")


class Model:
    def __init__(self, markov_model):
        self.m = markov_model

    def make_tweet(self, size=140, tries=100, max_overlap_ratio=0.5, **kw):
        return self.m.make_short_sentence(size, tries=tries,
                                          max_overlap_ratio=max_overlap_ratio,
                                          **kw)

    def dump(self, writer):
        d = {
            "version": DUMP_FMT_VERSION,
            # We can't store a dict because JSON doesn't accept lists as keys
            "chain": list(self.m.chain.model.items()),
        }
        return json.dump(d, writer)

    @classmethod
    def load(cls, reader):
        d = json.load(reader)
        assert d["version"] == DUMP_FMT_VERSION
        items = [(tuple(k), v) for k, v in d["chain"]]

        # .from_json also works on dicts
        chain = Chain.from_json(dict(items))
        return cls(NewlinePOSifiedText("", chain=chain))


class ModelBuilder:
    def __init__(self, path):
        self.path = path
        self._load()

    def _load(self):
        if self.path is not None and os.path.isfile(self.path):
            with open(self.path) as f:
                self._markov = Model.load(f).m
        else:
            self._markov = None

    def feed_corpus(self, text, newlines=True):
        """
        Feed a text corpus to the builder. If ``newlines`` is true (the
        default) then each line is assumed to be a sentence. We use this for
        tweets as they rarely have full sentences in them.

        It's more efficient to feed one big corpus than multiple small ones
        """
        cls = NewlinePOSifiedText if newlines else POSifiedText
        model = cls(text)

        # Avoid a call to 'combine' if this is the first corpus
        if self._markov is None:
            self._markov = model
            return

        self._markov = cls(markovify.combine([self._markov, model]))

    def model(self):
        if self._markov:
            return Model(self._markov)

    def save(self):
        model = self.model()
        if not model:
            return

        with open(self.path, "w") as f:
            model.dump(f)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.save()


class TwitterDigger:
    def __init__(self, cfg):
        names = cfg.get("seeds", "screen_names")
        self.names = deque([n.strip() for n in names.split(",")])
        self._seen = set()
        self._twitter = TwitterClient(cfg)
        self._lang = cfg.get("bot", "lang")
        self.logger = getLogger("digger")

    def screen_names(self, pick_friends=10):
        """
        Generate Twitter screen names from the configured seed list
        """
        while self.names:
            screen_name = self.names.popleft()
            if screen_name in self._seen:
                continue
            self._seen.add(screen_name)
            yield screen_name

            if pick_friends <= 0:
                continue

            for friend in self._twitter.friends(screen_name, n=pick_friends):
                if self._lang and friend.lang != self._lang:
                    continue
                if friend.protected or not friend.statuses_count:
                    continue
                self.names.append(friend.screen_name)

    def account_timeline(self, screen_name, size=1000):
        """
        Yield an account's last tweets. Tweets are filtered and cleaned up so
        the returned size might be lower than the one given as an argument
        (default is 1000).
        """
        acceptable_languages = set()
        if self._lang:
            # Accept tweets with unknown languages
            acceptable_languages = set((None, "und", self._lang))

        for status in self._twitter.user_tweets(screen_name, n=size):
            text = filter_status(status, acceptable_languages)
            if text:
                yield text

    def tweets(self, pick_friends=10, timeline_size=1000):
        """
        Generate tweets from seed accounts' timelines and their friends'.
        """
        self.logger.debug((
            "Retrieving tweets with options pick_friends=%d,"
            " timeline_size=%d") % (pick_friends, timeline_size))

        for name in self.screen_names(pick_friends=pick_friends):
            self.logger.debug("Checking @%s..." % name)
            for text in self.account_timeline(name, size=timeline_size):
                yield text

def filter_status(status, languages=None):
    if languages and status.lang not in languages:
        return

    if status.is_quote_status:
        return

    return filter_status_text(status.text)

def filter_status_text(text):
    text = tx.merge_spaces(text)
    text = tx.strip_urls(text)

    min_length = 30  # Arbitrary

    text = text.strip()
    # Don't yield empty or too short texts
    if len(text) < min_length:
        return

    lower_text = text.lower()
    for prefix in (
            # Global
            "rt ", "mt ", "@", ".@",
            # FR
            "je ", "moi ", "mon ", "ma ", "mes ",
            # EN
            "i ", "my "):
        if lower_text.startswith(prefix):
            return

    # truncated
    if text.endswith("…"):
        return

    # Those are mostly headlines
    if re.match(r"^[-\w]+ *:", text):
        return

    text = tx.normalize(text)

    # Re-check the text length now that it has been normalized
    if len(text) < min_length:
        return

    # Remove tweets that consist of hashtags/mentions only
    if [w for w in text.split(" ") if w.startswith("#") or w.startswith("@")]:
        return

    # Remove tweets with no text
    if re.match(r"^\W+$", text):
        return

    return text



# -*- coding: UTF-8 -*-

import logging

def getLogger(name):
    logger = logging.getLogger("blabbr.%s" % name)
    logger.setLevel(logging.DEBUG)

    if not logger.hasHandlers():
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        ch.setFormatter(logging.Formatter(
            '%(asctime)s [%(levelname)s] %(name)s: %(message)s'))

        logger.addHandler(ch)

    return logger



# -*- coding: UTF-8 -*-

import re

# From:
#   http://stackoverflow.com/a/33417311/735926
#   http://jrgraphix.net/r/Unicode/2600-26FF
#   https://en.wikipedia.org/wiki/Enclosed_Alphanumeric_Supplement
#   https://en.wikipedia.org/wiki/CJK_Symbols_and_Punctuation
#   https://en.wikipedia.org/wiki/Miscellaneous_Technical
#   https://en.wikipedia.org/wiki/Variation_Selectors_(Unicode_block)
EMOJI_RE = re.compile("["
    "\U00002300-\U000023FF"  # misc technical symbols
    "\U00002600-\U000026FF"  # misc symbols
    "\U00002700-\U000027BF"  # dingbats
    "\U00003000-\U0000303F"  # CJK symbols (Chinese/Japanese/Korean)
    "\U0000FE00-\U0000FE0F"  # variation selectors
    "\U0001F100-\U0001F1FF"  # enclosed chars
    "\U0001F1E0-\U0001F1FF"  # flags (iOS)
    "\U0001F300-\U0001F5FF"  # symbols & pictographs
    "\U0001F600-\U0001F64F"  # emoticons
    "\U0001F680-\U0001F6FF"  # transport & map symbols
    "]+")

def merge_spaces(text):
    return re.sub(r"\s+", " ", text)


def strip_urls(text):
    """Remove (truncated) URLs"""
    return re.sub(r"https?://[^ ]*", "", text)


def normalize(text):
    repls = (
        (r"&gt;", ">"),
        (r"&lt;", "<"),
        (r"&amp;", "&"),

        (EMOJI_RE, ""),

        # Treat quotes as parts of the tweet for more fun
        (r'["“”«»]', " "),

        # Remove random punctuation at the beginning & the end
        (r"^[-'&/\.,;><=]+", ""),
        (r"[-/><=:@&]+$", ""),

        # Lower-case what we're pretty sure are not accronyms
        (r"([A-ZÀÈÌÒÙÁÉÍÓÚÝÂÊÎÔÛÄËÏÖÜŸ]{6,})", lambda m: m.group(1).lower()),

        # "foo:" -> "foo :" (FR)
        (r"\b:", " :"),
        # "foo?" -> "foo ?" (FR)
        (r"\b\?", " ?"),
        # ".!!!" -> "!"
        (r"\.*!+\.*", "!"),
        # ".?!!!" -> "?!"
        (r"\.*\?+!+\.*", "?!"),
        # "foo!!" -> "foo !" (FR)
        (r"\b!+", " !"),
        # "25 %" -> "25%"
        (r"\b %", "%"),
        # "blabla ." -> "blabla."
        (r" \.", "."),

        (r" :: ", " "),
        (r" :- ", " "),

        (r"\b\.\.\b", "... "),

        # "[...]" -> "..."
        (r"\[\.\.\.+\]", "..."),
        # ":..." -> "..."
        (r":\.\.\.+", "..."),

        # Fix bogus ellipsis e.g. "...…" -> "…"
        (r"\.*…\.*", "… "),

        # Remove "via @foo"
        (r"via @[^ ]+$", ""),
        (r"(?:via )?#feedly", ""),
        (r"via$", ""),

        # "aa | bb" -> "aa  bb"
        (r"\|", ""),
    )

    for before, after in repls:
        text = re.sub(before, after, text).strip()

    typos = (
        # FR abbreviations & typos
        (r"\bpr\b", "pour"),
        (r"\bc est ", "c'est "),
        (r"\bs'est ", "c'est "),
    )

    for before, after in typos:
        text = re.sub(before, after, text, flags=re.IGNORECASE).strip()

    return merge_spaces(text)


def fix_punctuation(text):
    # This may vary per language
    text = re.sub(r" (['’]) ", "\\1", text)
    text = re.sub(r" ([\.,])", "\\1", text)

    return text



# -*- coding: UTF-8 -*-

import sys
import time

import tweepy
import tweepy.auth


class TwitterClient:
    @classmethod
    def __init__(self, cfg):
        ks = cfg.get_auth()

        auth = tweepy.OAuthHandler(ks["consumer_key"],
                                   ks["consumer_secret"])
        auth.set_access_token(ks["token"], ks["token_secret"])
        self.api = tweepy.API(auth)

    def verify_credentials(self):
        return self.api.verify_credentials()

    def user_tweets(self, user_id, n=20):
        return rate_limited_generator(self.api.user_timeline, n, id=user_id)

    def home_tweets(self, n=20):
        """
        Return up to ``n`` tweets from the bot's timeline.
        """
        return rate_limited_generator(self.api.home_timeline, n)

    def tweet(self, text):
        self.api.update_status(text)

    def friends(self, user_id, n=20):
        return rate_limited_generator(self.api.friends, n, id=user_id)


def rate_limited(cursor, sleeping_time=(15*60+5)):
    while True:
        try:
            yield next(cursor)
        except tweepy.RateLimitError:
            print("Sleeping...", file=sys.stderr)
            time.sleep(sleeping_time)

def rate_limited_generator(method, n, **kw):
    cursor = tweepy.Cursor(method, count=n, **kw)
    for el in rate_limited(cursor.items(n)):
        yield el



# -*- coding: UTF-8 -*-

import os.path
from configparser import ConfigParser

FEATURES = ("follow", "unfollow", "like", "retweet")

DEFAULT_CONFIG = {
    "bot": {
        "lang": "en",
        "timezone": "Europe/Paris",
    },
    "auth": {
        "consumer_key": "",
        "consumer_secret": "",
        "token": "",
        "token_secret": "",
    },
    "seeds": {
        "screen_names": "",
    }
}

for f in FEATURES:
    DEFAULT_CONFIG["bot"][f] = True


class Config:
    DEFAULT_PATH = "blabbr.cfg"  # current directory

    LOOKUP_PATHS = (
        DEFAULT_PATH,
        os.path.expanduser("~/.blabbr.cfg")
    )

    def __init__(self, path):
        self.path = path
        self.cfg = ConfigParser()
        self.load()

    @classmethod
    def default(cls):
        for path in cls.LOOKUP_PATHS:
            if os.path.isfile(path):
                return cls(path)

        return cls(cls.DEFAULT_PATH)

    @classmethod
    def from_path(cls, path):
        if path is not None:
            return cls(os.path.expanduser(path))

        return cls.default()

    def load(self):
        if not os.path.isfile(self.path):
            self.set_defaults()
            self.save()
        else:
            self.cfg.read(self.path)
            self.set_defaults()

    def save(self):
        with open(self.path, "w") as f:
            self.cfg.write(f)

    def set_defaults(self):
        for section, body in DEFAULT_CONFIG.items():
            if self.cfg.has_section(section):
                before = dict(self.cfg[section])
            else:
                self.cfg.add_section(section)
                before = {}

            merged = {}
            merged.update(body)
            merged.update(before)
            self.cfg[section] = merged

    def git_like_representation(self):
        lines = []
        for section, k, v in self:
            lines.append("%s.%s = %s" % (section, k, v))
        return "\n".join(lines)

    def get(self, section, name, **kw):
        return self.cfg.get(section, name, **kw)

    def set(self, section, name, value):
        self.cfg[section][name] = value

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.save()

    def __iter__(self):
        for section in self.cfg:
            for k, v in self.cfg[section].items():
                yield (section, k, v)

    def set_auth(self, auth):
        self.cfg["auth"] = dict(auth)

    def get_auth(self):
        return {k: v for k, v in self.cfg["auth"].items() if v}

    def enabled_features(self):
        feats = self.cfg["bot"]
        return {k: feats.getboolean(k, fallback=True) for k in FEATURES}



# -*- coding: UTF-8 -*-

import sys
import textwrap

import click
import tweepy

from blabbr.config import Config
from blabbr.model import ModelBuilder, TwitterDigger
from blabbr.twitter import TwitterClient
from blabbr.bot import Bot


class Cli:
    def __init__(self, cfg=None, model=None, **kw):
        self.cfg = Config.from_path(cfg)
        self.model_path = model
        self.model_builder = None

    def print_text(self, text, width=80, **kw):
        text = textwrap.dedent(text).strip()

        paragraphs = [
            textwrap.fill(p, width=width)
            for p in text.split("\n\n")]

        click.echo("\n\n".join(paragraphs), **kw)

    def setup(self, force=False, check=False, **kw):
        if check:
            return self.setup_check()

        did_setup = True
        did_setup &= self.setup_nltk(force=force, **kw)
        did_setup &= self.setup_auth(force=force, **kw)
        if not did_setup and not force:
            click.echo("The bot is already setup! Use --force if you're sure.",
                       err=True)
            sys.exit(1)

    def setup_auth(self, noninteractive=False, force=False, **kw):
        if not force and len(self.cfg.get_auth()) >= 4:
            return False

        auth = {k: kw.get(k) for k in
                ("consumer_key", "consumer_secret", "token", "token_secret")}

        missing_infos = set(k for k, v in auth.items() if v is None)

        if missing_infos:
            if noninteractive:
                click.echo("Missing auth info: %s" % ", ".join(missing_infos),
                           err=True)
                sys.exit(1)

            if missing_infos & {"consumer_key", "consumer_secret"}:
                self.print_text("""
                    You need to create a Twitter app in order to access the
                    Twitter API. Go on the following page:

                        https://apps.twitter.com/app/new

                    You don't have to fill the callback URL. Also, the account
                    with which you create your app has little importance as
                    long as you have a valid mobile phone number attached to
                    it.

                    Once you're done, go on the "Keys and Access Tokens" tab
                    and copy the consumer key & secret here.
                """)
                click.echo()

                auth["consumer_key"] = input("Consumer key: ").strip()
                auth["consumer_secret"] = input("Consumer secret: ").strip()
                click.echo()

            if missing_infos & {"token", "token_secret"}:
                oauth = tweepy.OAuthHandler(auth["consumer_key"],
                                            auth["consumer_secret"])
                redirect_url = oauth.get_authorization_url()

                self.print_text("""
                    You need to create an account for your bot then make it
                    allow your app to use its account.

                    Open this URL in a browser in which you're logged with your
                    bot's account. It'll give you a verification code you'll
                    copy back here.
                """)

                click.echo("\n    %s\n" % redirect_url)

                code = input("Verification code: ")
                oauth.get_access_token(code)
                auth["token"] = oauth.access_token
                auth["token_secret"] = oauth.access_token_secret
                click.echo()

        self.cfg.set_auth(auth)

        client = TwitterClient(self.cfg)
        valid_creds = client.verify_credentials()

        if valid_creds:
            click.echo("Your bot's authentication is set up!")
            click.echo("Screen name: %s" % valid_creds.screen_name)
            click.echo("Name: %s" % valid_creds.name)

            # Save the bot identification infos in the config for later use.
            # This saves us an API call when we need them; e.g. when filtering
            # the home timeline since it may contains some of the bot's tweets.
            self.cfg.set("bot", "screen_name", valid_creds.screen_name)
        else:
            click.echo(
                "Your authentication credentials seem invalid. Please check"
                " them again.")

        self.setup_check()
        self.cfg.save()
        return True

    def setup_nltk(self, **kw):
        import nltk
        from nltk.data import find

        tagger = "averaged_perceptron_tagger"

        try:
            find("taggers/%s" % tagger)
        except LookupError:
            click.echo("Downloading NTLK data (~2MB)...")
            nltk.download(tagger)
            return True

        return False

    def setup_check(self):
        client = TwitterClient(self.cfg)
        valid_creds = client.verify_credentials()

        if valid_creds:
            click.echo("Your bot's authentication is set up!")
            click.echo("Screen name: %s" % valid_creds.screen_name)
            click.echo("Name: %s" % valid_creds.name)

            # Save the bot identification infos in the config for later use.
            # This saves us an API call when we need them; e.g. when filtering
            # the home timeline since it may contains some of the bot's tweets.
            self.cfg.set("bot", "screen_name", valid_creds.screen_name)
            self.cfg.set("bot", "id", valid_creds.id_str)
        else:
            click.echo(
                "Your authentication credentials seem invalid. Please check"
                " them again.")

    def config(self, name=None, value=None):
        if not name:
            click.echo(self.cfg.git_like_representation())
            return

        if name == "init":
            self.cfg.save()
            return

        if "." not in name:
            click.echo("Config variable must be of the form <section>.<name>",
                       err=True)
            sys.exit(1)

        section, name = name.split(".", 1)
        if value:
            self.cfg.set(section, name, value)
            self.cfg.save()
            return

        _nil = object()
        value = self.cfg.get(section, name, fallback=_nil)
        if value is not _nil:
            click.echo(value)

    def _load_model(self):
        self.model_builder = ModelBuilder(self.model_path)
        return self.model_builder

    def _model(self):
        return self.model_builder.model()

    def populate(self, raw=None, from_raw=None, pick_friends=10,
            timeline_size=1000):
        digger = TwitterDigger(self.cfg)

        tweets = []

        if from_raw:
            with open(from_raw) as f:
                for line in f:
                    tweets.append(line.rstrip())
        else:
            tweets = digger.tweets(pick_friends=pick_friends,
                                   timeline_size=timeline_size)

        if raw:
            with open(raw, "a") as f:
                try:
                    for tweet in tweets:
                        f.write("%s\n" % tweet.replace("\n", " "))
                except KeyboardInterrupt:
                    pass
            return

        return self._populate(tweets)

    def _populate(self, tweets):
        chunk_size = 2000

        with self._load_model() as mb:
            corpus = []
            try:
                for tweet in tweets:
                    corpus.append(tweet)
                    if len(corpus) == chunk_size:
                        click.echo("Feeding %d tweets..." % chunk_size)
                        mb.feed_corpus("\n".join(corpus))
                        corpus = []
            except KeyboardInterrupt:
                pass
            if corpus:
                click.echo("Feeding %d tweets..." % len(corpus))
                mb.feed_corpus("\n".join(corpus))

    def run(self, dry_run=False, debug=False):
        self._load_model()
        model = self._model()
        if model is None:
            raise RuntimeError("The bot cannot run with an empty model")

        Bot(cfg=self.cfg, model=model, dry_run=dry_run, debug=debug).live()


@click.group()
@click.option('--cfg', type=click.Path(), help="Path to the config")
@click.option("--model", type=click.Path(),
              default="blabbr.json",  # default to the current directory
              help="Path to the saved model")
@click.pass_context
def cli(ctx, **kw):
    ctx.obj = Cli(**kw)


@cli.command()
@click.option("--consumer-key", metavar="KEY", help="Consumer key (API key)")
@click.option("--consumer-secret", metavar="SECRET",
              help="Consumer secret (API secret)")
@click.option("--token", metavar="TOKEN", help="Access token")
@click.option("--token-secret", metavar="SECRET", help="Access token secret")
@click.option("--noninteractive", is_flag=True,
              help=("Fail if one option is missing instead of offering an"
                    " interactive setup."))
@click.option("--force", is_flag=True,
              help="Force the setup even if the bot is already set.")
@click.option("--check", is_flag=True,
              help="Check the authentication setup.")
@click.pass_obj
def setup(cli, *args, **kw):
    """Setup the bot's config"""
    cli.setup(*args, **kw)


@cli.command()
@click.option("--raw", type=click.Path(),
              help=("Dump tweets in a file instead of feeding a model."
                    " Useful for debugging."))
@click.option("--from-raw", type=click.Path(),
              help=("Read tweets from a file instead of the Twitter API."))
@click.option("--pick-friends", type=int,
              default=10,
              help=("Number of friends to check when populating from Twitter"
                    " (default: 10)."))
@click.option("--timeline-size", type=int,
              default=1000,
              help="Number of tweets to retrieve per user (default: 1000)")
@click.pass_obj
def populate(cli, *args, **kw):
    """Populate the Markov model"""
    cli.populate(*args, **kw)


@cli.command()
@click.option("--dry-run", "-n", is_flag=True,
              help="Don't tweet anything (useful for debugging).")
@click.option("--debug", is_flag=True,
        help="Debug mode. Implies --dry-run.")
@click.pass_obj
def run(cli, *args, **kw):
    """Run the bot"""
    cli.run(*args, **kw)


@cli.command()
@click.argument("name", required=False)
@click.argument("value", required=False)
@click.pass_obj
def config(cli, *args, **kw):
    """
    Manage the bot's configuration. This command works like `git config` and
    prints the whole configuration when called without argument. Passing a name
    prints only this variable, and passing both a name and a value sets the
    variable.
    """
    cli.config(*args, **kw)


if __name__ == "__main__":
    cli()



# -*- coding: UTF-8 -*-

from setuptools import setup

# http://stackoverflow.com/a/7071358/735926
import re
VERSIONFILE='blabbr/__init__.py'
verstrline = open(VERSIONFILE, 'rt').read()
VSRE = r'^__version__\s+=\s+[\'"]([^\'"]+)[\'"]'
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % VERSIONFILE)

setup(
    name='blabbr',
    version=verstr,
    author='Baptiste Fontaine',
    author_email='b@ptistefontaine.fr',
    packages=['blabbr'],
    url='https://github.com/bfontaine/blabbr',
    license=open('LICENSE', 'r').read(),
    description='Twitter bot',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    entry_points={
        'console_scripts':[
            'blabbr = blabbr.cli:cli',
        ]
    },
)



# -*- coding: UTF-8 -*-

import time
from random import randint, random
from datetime import timedelta

from blabbr.generation import Generator
from blabbr.twitter import TwitterClient
from blabbr.logging import getLogger

from blabbr.time import Clock

MIN_TWEET_INTERVAL = timedelta(minutes=20)


class Bot:
    """
    A Twitter bot.
    """

    def __init__(self, cfg=None, generator=None, clock=None, model=None,
                 dry_run=False, debug=False):
        self.twitter = TwitterClient(cfg=cfg)
        self.clock = Clock(cfg) if clock is None else clock
        self.logger = getLogger("bot")

        if generator:
            self.generator = generator
        elif model:
            self.generator = Generator(model)
        else:
            raise RuntimeError("The bot needs a generator or a model")

        if debug:
            dry_run = True

        self.features = cfg.enabled_features()
        self.dry_run = dry_run
        self.debug = debug
        self.last_tweet = None
        self.last_tweet_time = None

    def live(self):
        """
        Start the bot's life. This methods never returns but may raise
        exceptions.
        """
        self.logger.debug("Starting to live...")
        self.logger.debug("Dry run: %s" % self.dry_run)

        while True:
            if self.clock.time_to_sleep():
                self.logger.debug("Sleeping...")
                self.sleep(40, 50)
                continue

            if not self.clock.time_to_chill():
                self.logger.debug("Not a time to chill...")
                self.sleep(20, 25)
                continue

            self.tick()
            self.sleep(1, 2)

    def tick(self):
        """
        This method is called every one to two minutes when the bot has free
        time.
        """
        self.logger.debug("tick")
        feeling_inspired = random() > 0.7
        if not feeling_inspired:
            self.logger.debug("Not feeling inspired")
            return

        self.tweet()

    def tweet(self):
        """
        Post a random tweet. This has no effect if the last tweet was less than
        20s ago or if the ``tweet`` feature is disabled.
        """
        if not self.features.get("tweet", True):
            self.logger.debug("Tweeting is disabled in the config")
            return

        now = self.clock.now()
        if self.last_tweet_time and \
                now < self.last_tweet_time + MIN_TWEET_INTERVAL:
            return

        text = self.generator.tweet()
        self.logger.debug("About to tweet: %s" % text)
        if not self.dry_run:
            self.twitter.tweet(text)

        self.last_tweet = text
        self.last_tweet_time = now

    def sleep(self, minutes_min, minutes_max):
        """
        Sleep a random time between the given number of minutes.
        """
        duration = randint(minutes_min * 60, minutes_max * 60)
        if self.debug:
            self.logger.debug("Would sleep %d seconds" % duration)
        else:
            time.sleep(duration)


if __name__ == "__main__":
    Bot().live()
